/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.prototypes;

import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createActuator;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createActuatorOut;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createBusMasterInterface;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createBusMasterPort;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createCANBus;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createCANBusMasterInterface;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createCANBusMasterPort;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createRaspberryPi3;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createSensor;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createSensorIn;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.setConnectorPosition;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setNodePosition;

import org.fortiss.af3.platform.raspberry.model.Actuator;
import org.fortiss.af3.platform.raspberry.model.ActuatorOut;
import org.fortiss.af3.platform.raspberry.model.BusMasterInterface;
import org.fortiss.af3.platform.raspberry.model.BusMasterPort;
import org.fortiss.af3.platform.raspberry.model.CANBus;
import org.fortiss.af3.platform.raspberry.model.CANBusMasterInterface;
import org.fortiss.af3.platform.raspberry.model.CANBusMasterPort;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi3;
import org.fortiss.af3.platform.raspberry.model.Sensor;
import org.fortiss.af3.platform.raspberry.model.SensorIn;
import org.fortiss.tooling.kernel.extension.base.PrototypeProviderBase;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class PrototypeProvider extends PrototypeProviderBase {

	/** Stores the category to display for these prototypes */
	protected final static String CATEGORY_NAME = "Raspberry Platform";

	/** {@inheritDoc} */
	@Override
	protected void registerPrototypes() {
		setupRaspberryPlatform();
	}

	/**
	 * 
	 */
	private void setupRaspberryPlatform() {
		Actuator actuator = createActuator("Actuator");
		setNodePosition(actuator, 0, 0);
		setConnectorPosition(actuator, 0, 0);
		registerPrototype("Actuator", actuator, CATEGORY_NAME);

		ActuatorOut actuatorOut = createActuatorOut("ActuatorOut");
		setNodePosition(actuatorOut, 0, 0);
		setConnectorPosition(actuator, 0, 0);
		registerPrototype("ActuatorOut", actuatorOut, CATEGORY_NAME);

		BusMasterInterface busMasterInterface = createBusMasterInterface("BusMasterInterface");
		setNodePosition(busMasterInterface, 0, 0);
		setConnectorPosition(busMasterInterface, 0, 0);
		registerPrototype("BusMasterInterface", busMasterInterface, CATEGORY_NAME);

		BusMasterPort busMasterPort = createBusMasterPort("BusMasterPort");
		setNodePosition(busMasterPort, 0, 0);
		setConnectorPosition(busMasterPort, 0, 0);
		registerPrototype("BusMasterPort", busMasterPort, CATEGORY_NAME);

		CANBus canBus = createCANBus("CANBus");
		setNodePosition(canBus, 0, 0);
		setConnectorPosition(canBus, 0, 0);
		registerPrototype("CANBus", canBus, CATEGORY_NAME);

		CANBusMasterInterface canBusMasterInterface =
				createCANBusMasterInterface("CANBusMasterInterface");
		setNodePosition(canBusMasterInterface, 0, 0);
		setConnectorPosition(canBusMasterInterface, 0, 0);
		registerPrototype("CANBusMasterInterface", canBusMasterInterface, CATEGORY_NAME);

		CANBusMasterPort canBusMasterPort = createCANBusMasterPort("CANBusMasterPort");
		setNodePosition(canBusMasterPort, 0, 0);
		setConnectorPosition(canBusMasterPort, 0, 0);
		registerPrototype("CANBusMasterPort", canBusMasterPort, CATEGORY_NAME);

		RaspberryPi3 raspberryPi3 = createRaspberryPi3("RaspberryPi3");
		setNodePosition(raspberryPi3, 0, 0);
		setConnectorPosition(raspberryPi3, 0, 0);
		registerPrototype("RaspberryPi3", raspberryPi3, CATEGORY_NAME);

		Sensor sensor = createSensor("Sensor");
		setNodePosition(sensor, 0, 0);
		setConnectorPosition(sensor, 0, 0);
		registerPrototype("Sensor", sensor, CATEGORY_NAME);

		SensorIn sensorIn = createSensorIn("SensorIn");
		setNodePosition(sensorIn, 0, 0);
		setConnectorPosition(sensorIn, 0, 0);
		registerPrototype("SensorIn", sensorIn, CATEGORY_NAME);
	}

}
