/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.deployment.utils.DeploymentUtils.findECUforDeployedComponent;
import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.createVar;
import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.funcCall;
import static org.fortiss.tooling.base.utils.AnnotationUtils.getAnnotation;

import org.eclipse.emf.common.util.EList;
import org.fortiss.af3.component.model.Channel;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.component.model.Port;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.expression.model.terms.IExpressionTerm;
import org.fortiss.af3.platform.language.executable.TransceiverExecutableBase;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.IPlatformArchitectureElement;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.IReadExecutableWithNoValSupport;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.ITermExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.IWriteExecutableWithNoValSupport;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.InitExecutable;
import org.fortiss.af3.platform.raspberry.model.BusMasterPort;
import org.fortiss.af3.platform.raspberry.model.annotation.CoreAddress;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class BusMasterPortExecutable extends TransceiverExecutableBase<BusMasterPort> implements
		InitExecutable, ITermExecutable, IReadExecutableWithNoValSupport,
		IWriteExecutableWithNoValSupport {

	/**
	 * @param modelElement
	 */
	public BusMasterPortExecutable(BusMasterPort modelElement) {
		super(modelElement);
		// TODO Auto-generated constructor stub
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(Deployment depl) {
		int currentCoreAddress = getParentCoreAddress();
		return funcCall("init_BusMasterPort", createVar(currentCoreAddress + ""));// +
																					// modelElement.getName()
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(Deployment depl) {
		int currentCoreAddress = getParentCoreAddress();
		return funcCall("term_BusMasterPort", createVar(currentCoreAddress + ""));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getValueWriteAccessor(OutputPort logicalSignal, IExpressionTerm value,
			Deployment depl) {
		int coreAddress = getTargetCoreAddress(logicalSignal, depl);
		String signalId = logicalSignal.getName() + "_ID_" + logicalSignal.getId();
		return funcCall("write_BusMasterPort", createVar(coreAddress + ""), createVar(signalId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getValueReadAccessor(InputPort logicalSignal, Deployment depl) {
		Port sourceSignal = getSourceSignal(logicalSignal);
		String signalId = sourceSignal.getName() + "_ID_" + sourceSignal.getId();
		return funcCall("read_BusMasterPort", createVar(signalId));

	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValWriteAccessor(OutputPort logicalSignal) {
		return funcCall("set_noval_BusMasterPort");
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValGuardAccessor(InputPort logicalSignal) {
		return funcCall("is_noval_BusMasterPort");
	}

	/** Get source signal for the current port */
	private Port getSourceSignal(InputPort logicalSignal) {
		EList<Channel> channels = logicalSignal.getIncomingChannels();
		// get one of the channels, usually only one channel will be connected per input port
		Port sourceSignal = channels.get(0).getSource();
		return sourceSignal;
	}

	/** Gets the address for the core that acts as sink for the Output Port */
	private int getTargetCoreAddress(OutputPort logicalSignal, Deployment depl) {
		EList<Channel> channels = logicalSignal.getOutgoingChannels();
		// get one of the channels, usually only one channel will be connected per output port
		Component targetComponent = channels.get(0).getTarget().getComponent();
		ExecutionUnit targetCore = findECUforDeployedComponent(targetComponent, depl);
		CoreAddress ias = getAnnotation(targetCore, CoreAddress.class);
		return ias.getCoreAddress();
	}

	/** Get core address the modelElement is deployed on */
	private int getParentCoreAddress() {
		IPlatformArchitectureElement parentElement = modelElement.getPlatformElement();
		CoreAddress ias = getAnnotation(parentElement, CoreAddress.class);
		return ias.getCoreAddress();
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getValueReadAccessor(InputPort logicalSignal) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValGuardAccessor(IExpressionTerm signal, IExpressionTerm buffer) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getValueWriteAccessor(OutputPort logicalSignal, IExpressionTerm value) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination() {
		// TODO Auto-generated method stub
		return funcCall("term_BusMasterPort");
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization() {
		// TODO Auto-generated method stub
		return funcCall("init_BusMasterPort");
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(IExpressionTerm signal) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm term) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm port, IExpressionTerm portName,
			IExpressionTerm readOrWrite) {
		// TODO Auto-generated method stub
		return null;
	}

}
