/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.createVar;
import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.funcCall;
import static org.fortiss.tooling.base.utils.AnnotationUtils.getAnnotation;

import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.expression.model.terms.IExpressionTerm;
import org.fortiss.af3.platform.language.executable.ReceiverExecutableBase;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.IReadExecutableWithNoValSupport;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.ITermExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.InitExecutable;
import org.fortiss.af3.platform.raspberry.model.SensorIn;
import org.fortiss.af3.platform.raspberry.model.annotation.PeripheralAddress;
import org.fortiss.af3.platform.raspberry.model.annotation.PinNumber;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class SensorInExecutable extends ReceiverExecutableBase<SensorIn> implements InitExecutable,
		ITermExecutable, IReadExecutableWithNoValSupport {

	/**
	 * @param modelElement
	 */
	public SensorInExecutable(SensorIn modelElement) {
		super(modelElement);
		// TODO Auto-generated constructor stub
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization() {
		String sensorId = "SensorIn_ID_" + modelElement.getId();// modelElement.getName() +
		return funcCall("init_Sensor", createVar(sensorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination() {
		String sensorId = "SensorIn_ID_" + modelElement.getId();
		return funcCall("term_Sensor", createVar(sensorId));
	}

	/** {@inheritDoc} */
	@Override
	/* generates call: read_Sensor(pinNr, varName) */
	public IExpressionTerm getValueReadAccessor(InputPort logicalSignal) {
		// PinNumber ias = getAnnotation(modelElement, PinNumber.class);
		// int pinNr = ias.getPinNumber();
		String sensorId = "SensorIn_ID_" + modelElement.getId();
		String fcnName = "read_Sensor";
		return funcCall(fcnName, createVar(sensorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValGuardAccessor(InputPort logicalSignal) {
		return funcCall("is_noval_Sensor");
	}

	/** {@inheritDoc} */
	@Override
	/* generates call: read_Sensor(id, pinNr, address) */
	public IExpressionTerm getValueReadAccessor(InputPort logicalSignal, Deployment depl) {
		String sensorId = "SensorIn_ID_" + modelElement.getId();
		String fcnName = "read_Sensor";
		return funcCall(fcnName, createVar(sensorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValGuardAccessor(IExpressionTerm signal, IExpressionTerm buffer) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(IExpressionTerm signal) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(Deployment depl) {
		// TODO Auto-generated method stub
		String sensorId = "SensorIn_ID_" + modelElement.getId();
		return funcCall("term_Sensor", createVar(sensorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm term) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm port, IExpressionTerm portName,
			IExpressionTerm readOrWrite) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(Deployment depl) {
		// TODO Auto-generated method stub
		String sensorId = "SensorIn_ID_" + modelElement.getId();
		return funcCall("init_Sensor", createVar(sensorId));
	}

	/** Get PinNr, Address and ID of a sensor. */

	public String getSensorDetails() {
		PinNumber ias = getAnnotation(modelElement, PinNumber.class);
		int pinNr = ias.getPinNumber();
		PeripheralAddress pa = getAnnotation(modelElement, PeripheralAddress.class);
		int sensorAddress = pa.getPeripheralAddress();
		int sensorId = modelElement.getId();
		return "{" + sensorId + ", " + pinNr + ", " + sensorAddress + "} \n";
	}

}
