/*--------------------------------------------------------------------------+
$Id: PikeosPartitionExecutable.java 10808 2014-05-27 09:00:26Z zverlov $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

package org.fortiss.af3.platform.raspberry.generator.executable.interfaces;

import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.expression.model.terms.IExpressionTerm;

/**
 * Interface implemented by platform-specific executables, which need
 * initialization during system start-up.
 * 
 * @author chaudhary
 * @author $Author: chaudhary $
 * @version $Rev: 10810 $
 * @ConQAT.Rating GREEN Hash: E1C77D8E8AD1D2AA9F058E3034D6B88E
 */
public interface InitExecutable {

	/**
	 * Returns the expression term for ports initialization.
	 * 
	 * @param term
	 *            the expression term
	 * @return expression term for ports initialization
	 */
	public abstract IExpressionTerm getInitialization(IExpressionTerm term);

	/**
	 * Returns the expression term for ports initialization.
	 * 
	 * @param port
	 *            the port which needs to be initialized
	 * @param portName
	 *            name of the port
	 * @param readOrWrite
	 *            input or output port
	 * @return expression term for ports initialization
	 */
	public abstract IExpressionTerm getInitialization(IExpressionTerm port,
			IExpressionTerm portName, IExpressionTerm readOrWrite);

	/**
	 * @param depl
	 * @return
	 */
	IExpressionTerm getInitialization(Deployment depl);

	/**
	 * @return
	 */
	IExpressionTerm getInitialization();
}
